// This file is autogenerated. Changes will be overwritten.
#include "EWIEGA46WW/moc_AbstractSession.cpp"
#include "EWIEGA46WW/moc_Clipboard.cpp"
#include "EWIEGA46WW/moc_PlasmaScreencastV1Session.cpp"
#include "EWIEGA46WW/moc_screencasting_p.cpp"
