/****************************************************************************
** Meta object code from reading C++ file 'xdp_dbus_remotedesktop_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "xdp_dbus_remotedesktop_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xdp_dbus_remotedesktop_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopPortalRemoteDesktopInterface::qt_create_metaobjectdata<qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopPortalRemoteDesktopInterface",
        "ConnectToEIS",
        "QDBusPendingReply<QDBusUnixFileDescriptor>",
        "",
        "QDBusObjectPath",
        "session_handle",
        "QVariantMap",
        "options",
        "CreateSession",
        "QDBusPendingReply<QDBusObjectPath>",
        "NotifyKeyboardKeycode",
        "QDBusPendingReply<>",
        "keycode",
        "state",
        "NotifyKeyboardKeysym",
        "keysym",
        "NotifyPointerAxis",
        "dx",
        "dy",
        "NotifyPointerAxisDiscrete",
        "axis",
        "steps",
        "NotifyPointerButton",
        "button",
        "NotifyPointerMotion",
        "NotifyPointerMotionAbsolute",
        "stream",
        "x",
        "y",
        "NotifyTouchDown",
        "slot",
        "NotifyTouchMotion",
        "NotifyTouchUp",
        "SelectDevices",
        "Start",
        "parent_window",
        "AvailableDeviceTypes",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ConnectToEIS'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusUnixFileDescriptor>(const QDBusObjectPath &, const QVariantMap &)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'CreateSession'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QVariantMap &)>(8, 3, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'NotifyKeyboardKeycode'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, int, uint)>(10, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Int, 12 }, { QMetaType::UInt, 13 },
        }}),
        // Slot 'NotifyKeyboardKeysym'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, int, uint)>(14, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Int, 15 }, { QMetaType::UInt, 13 },
        }}),
        // Slot 'NotifyPointerAxis'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, double, double)>(16, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Double, 17 }, { QMetaType::Double, 18 },
        }}),
        // Slot 'NotifyPointerAxisDiscrete'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, uint, int)>(19, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::UInt, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'NotifyPointerButton'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, int, uint)>(22, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Int, 23 }, { QMetaType::UInt, 13 },
        }}),
        // Slot 'NotifyPointerMotion'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, double, double)>(24, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Double, 17 }, { QMetaType::Double, 18 },
        }}),
        // Slot 'NotifyPointerMotionAbsolute'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, uint, double, double)>(25, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::UInt, 26 }, { QMetaType::Double, 27 },
            { QMetaType::Double, 28 },
        }}),
        // Slot 'NotifyTouchDown'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, uint, uint, double, double)>(29, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::UInt, 26 }, { QMetaType::UInt, 30 },
            { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
        }}),
        // Slot 'NotifyTouchMotion'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, uint, uint, double, double)>(31, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::UInt, 26 }, { QMetaType::UInt, 30 },
            { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
        }}),
        // Slot 'NotifyTouchUp'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &, uint)>(32, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::UInt, 30 },
        }}),
        // Slot 'SelectDevices'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QDBusObjectPath &, const QVariantMap &)>(33, 3, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'Start'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QDBusObjectPath &, const QString &, const QVariantMap &)>(34, 3, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 35 }, { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'AvailableDeviceTypes'
        QtMocHelpers::PropertyData<uint>(36, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'version'
        QtMocHelpers::PropertyData<uint>(37, QMetaType::UInt, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopPortalRemoteDesktopInterface, qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopPortalRemoteDesktopInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopPortalRemoteDesktopInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopPortalRemoteDesktopInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusPendingReply<QDBusUnixFileDescriptor> _r = _t->ConnectToEIS((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusUnixFileDescriptor>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<QDBusObjectPath> _r = _t->CreateSession((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->NotifyKeyboardKeycode((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->NotifyKeyboardKeysym((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->NotifyPointerAxis((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->NotifyPointerAxisDiscrete((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->NotifyPointerButton((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->NotifyPointerMotion((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->NotifyPointerMotionAbsolute((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->NotifyTouchDown((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->NotifyTouchMotion((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->NotifyTouchUp((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QDBusObjectPath> _r = _t->SelectDevices((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QDBusObjectPath> _r = _t->Start((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->availableDeviceTypes(); break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->version(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopPortalRemoteDesktopInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopPortalRemoteDesktopInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgFreedesktopPortalRemoteDesktopInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopPortalRemoteDesktopInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
