/****************************************************************************
** Meta object code from reading C++ file 'krdpserversettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../krdpserversettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'krdpserversettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18KRDPServerSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto KRDPServerSettings::qt_create_metaobjectdata<qt_meta_tag_ZN18KRDPServerSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KRDPServerSettings",
        "ListenPortChanged",
        "",
        "AutogenerateCertificatesChanged",
        "CertificateChanged",
        "CertificateKeyChanged",
        "QualityChanged",
        "UsersChanged",
        "SystemUserEnabledChanged",
        "AutostartChanged",
        "listenPort",
        "isListenPortImmutable",
        "defaultListenPortValue",
        "autogenerateCertificates",
        "isAutogenerateCertificatesImmutable",
        "defaultAutogenerateCertificatesValue",
        "certificate",
        "isCertificateImmutable",
        "certificateKey",
        "isCertificateKeyImmutable",
        "quality",
        "isQualityImmutable",
        "defaultQualityValue",
        "users",
        "isUsersImmutable",
        "systemUserEnabled",
        "isSystemUserEnabledImmutable",
        "defaultSystemUserEnabledValue",
        "autostart",
        "isAutostartImmutable",
        "defaultAutostartValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ListenPortChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AutogenerateCertificatesChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CertificateChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CertificateKeyChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'QualityChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'UsersChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SystemUserEnabledChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AutostartChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'listenPort'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isListenPortImmutable'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultListenPortValue'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'autogenerateCertificates'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isAutogenerateCertificatesImmutable'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultAutogenerateCertificatesValue'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificate'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isCertificateImmutable'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateKey'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isCertificateKeyImmutable'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'quality'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isQualityImmutable'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultQualityValue'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'users'
        QtMocHelpers::PropertyData<QStringList>(23, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isUsersImmutable'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'systemUserEnabled'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isSystemUserEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSystemUserEnabledValue'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'autostart'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'isAutostartImmutable'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultAutostartValue'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KRDPServerSettings, qt_meta_tag_ZN18KRDPServerSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KRDPServerSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KRDPServerSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KRDPServerSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18KRDPServerSettingsE_t>.metaTypes,
    nullptr
} };

void KRDPServerSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KRDPServerSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ListenPortChanged(); break;
        case 1: _t->AutogenerateCertificatesChanged(); break;
        case 2: _t->CertificateChanged(); break;
        case 3: _t->CertificateKeyChanged(); break;
        case 4: _t->QualityChanged(); break;
        case 5: _t->UsersChanged(); break;
        case 6: _t->SystemUserEnabledChanged(); break;
        case 7: _t->AutostartChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::ListenPortChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::AutogenerateCertificatesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::CertificateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::CertificateKeyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::QualityChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::UsersChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::SystemUserEnabledChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerSettings::*)()>(_a, &KRDPServerSettings::AutostartChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->listenPort(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isListenPortImmutable(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->defaultListenPortValue(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->autogenerateCertificates(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isAutogenerateCertificatesImmutable(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->defaultAutogenerateCertificatesValue(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->certificate(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isCertificateImmutable(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->certificateKey(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isCertificateKeyImmutable(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->quality(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isQualityImmutable(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->defaultQualityValue(); break;
        case 13: *reinterpret_cast<QStringList*>(_v) = _t->users(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isUsersImmutable(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->systemUserEnabled(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isSystemUserEnabledImmutable(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->defaultSystemUserEnabledValue(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->autostart(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isAutostartImmutable(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->defaultAutostartValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setListenPort(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setAutogenerateCertificates(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setCertificate(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setCertificateKey(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setQuality(*reinterpret_cast<int*>(_v)); break;
        case 13: _t->setUsers(*reinterpret_cast<QStringList*>(_v)); break;
        case 15: _t->setSystemUserEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setAutostart(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KRDPServerSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KRDPServerSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KRDPServerSettingsE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int KRDPServerSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void KRDPServerSettings::ListenPortChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KRDPServerSettings::AutogenerateCertificatesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KRDPServerSettings::CertificateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KRDPServerSettings::CertificateKeyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KRDPServerSettings::QualityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KRDPServerSettings::UsersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KRDPServerSettings::SystemUserEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KRDPServerSettings::AutostartChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
